var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import fs, { createReadStream } from 'fs';
import path from 'path';
import fetch from 'node-fetch';
import unzipper from 'unzipper';
import { sync as which } from 'which';
import { makeExecutable, writeFileToFolder } from 'myst-cli-utils';
import chalk from 'chalk';
const JATS_VERSIONS = ['1.2', '1.2d1', '1.2d2', '1.3', '1.3d1', '1.3d2'];
const DEFAULT_JATS_VERSION = '1.3';
const MATHML_VERSIONS = ['2', '3'];
const DEFAULT_MATHML_VERSION = '3';
function validateOptions(opts) {
    var _a;
    let jats;
    if (!opts.jats) {
        jats = DEFAULT_JATS_VERSION;
    }
    else if (!JATS_VERSIONS.includes(opts.jats)) {
        throw new Error(`Invalid JATS version "${opts.jats}" - must be one of [${JATS_VERSIONS.join(', ')}]`);
    }
    else {
        jats = opts.jats;
    }
    let mathml;
    if (!opts.mathml) {
        mathml = DEFAULT_MATHML_VERSION;
    }
    else if (!MATHML_VERSIONS.includes(opts.mathml)) {
        throw new Error(`Invalid MathML version "${opts.mathml}" - must be one of [${MATHML_VERSIONS.join(', ')}]`);
    }
    else {
        mathml = opts.mathml;
    }
    const out = {
        jats,
        mathml,
        oasis: !!opts.oasis,
        directory: (_a = opts.directory) !== null && _a !== void 0 ? _a : defaultDirectory(),
    };
    return out;
}
function dtdFolder(opts) {
    const version = opts.jats.replace('.', '-');
    const oasis = opts.oasis ? '-OASIS' : '';
    const mathml = `MathML${opts.mathml}`;
    return `JATS-Archiving-${version}${oasis}-${mathml}-DTD`;
}
function dtdZipFile(opts) {
    return `${dtdFolder(opts)}.zip`;
}
function localDtdZipFile(opts) {
    return path.join(opts.directory, dtdZipFile(opts));
}
function dtdFile(opts) {
    const version = opts.jats.startsWith('1.2') ? '1' : opts.jats.replace('.', '-');
    const article = opts.oasis ? 'archive-oasis-article' : 'archivearticle';
    const mathml = opts.mathml === '3' ? '-mathml3' : '';
    return `JATS-${article}${version}${mathml}.dtd`;
}
function localDtdFile(opts) {
    return path.join(opts.directory, dtdFolder(opts), dtdFile(opts));
}
function ftpUrl(opts) {
    return `https://ftp.ncbi.nih.gov/pub/jats/archiving/${opts.jats}/${dtdZipFile(opts)}`;
}
function defaultDirectory() {
    return path.join(__dirname, 'static');
}
function dtdDownload(session, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!fs.existsSync(opts.directory)) {
            fs.mkdirSync(opts.directory, { recursive: true });
        }
        session.log.info(`🌎 Downloading: ${ftpUrl(opts)}`);
        session.log.debug(`Saving to ${localDtdZipFile(opts)}`);
        const resp = yield fetch(ftpUrl(opts));
        writeFileToFolder(localDtdZipFile(opts), yield resp.buffer());
    });
}
function ensureDtdZipExists(session, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!fs.existsSync(path.join(opts.directory, dtdZipFile(opts)))) {
            yield dtdDownload(session, opts);
        }
    });
}
function ensureDtdExists(session, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!fs.existsSync(localDtdFile(opts))) {
            yield ensureDtdZipExists(session, opts);
            const zipFile = localDtdZipFile(opts);
            session.log.info(`🤐 Unzipping template on disk ${zipFile}`);
            yield createReadStream(zipFile)
                .pipe(unzipper.Extract({ path: opts.directory }))
                .promise();
        }
        session.log.debug(`Validating against ${localDtdFile(opts)}`);
    });
}
function isXmllintAvailable() {
    return which('xmllint', { nothrow: true });
}
export function validateJatsAgainstDtd(session, file, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!isXmllintAvailable()) {
            session.log.error(`JATS validation against DTD requires xmllint\n\n${chalk.dim('To install:\n  mac:    brew install xmlstarlet\n  debian: apt install libxml2-utils')}`);
            return;
        }
        const validatedOpts = validateOptions(opts !== null && opts !== void 0 ? opts : {});
        yield ensureDtdExists(session, validatedOpts);
        try {
            yield makeExecutable(`xmllint --dtdvalid ${localDtdFile(validatedOpts)} --noout ${file}`, session.log)();
        }
        catch (_a) {
            return false;
        }
        return true;
    });
}
export function validateJatsAgainstDtdWrapper(session, file, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const success = yield validateJatsAgainstDtd(session, file, opts);
        if (success) {
            session.log.info(chalk.greenBright('JATS validation passed!'));
        }
        else {
            throw new Error('JATS validation failed.');
        }
    });
}
